package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceUtil;

public class LoadStateAndDetachEntityExample2 {
    static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");
    static EntityManager entityManager = entityManagerFactory.createEntityManager();

    public static void main(String[] args) {
        try {
            persistEmployee();
            Employee employee = findEmployeeById(1);
            displayInGui(employee);
        } finally {
            entityManagerFactory.close();
        }
    }

    private static void displayInGui(Employee employee) {
        employee = checkAndInitialize(employee);
        System.out.println("-- displaying employee --");
        System.out.println(employee);
        employee.getPhones()
                .forEach(System.out::println);
    }

    private static Employee checkAndInitialize(Employee employee) {
        PersistenceUtil pu = entityManagerFactory.getPersistenceUnitUtil();
        System.out.println("       Employee loaded: " + pu.isLoaded(employee));
        System.out.println("Employee.phones loaded: " + pu.isLoaded(employee, "phones"));
        if (!pu.isLoaded(employee) //entity might've been retrieved via getReference
                || !pu.isLoaded(employee, "phones")//phones is a lazy relation
                ) {
            System.out.println("initializing employee");
            boolean detached = !entityManager.contains(employee);
            System.out.println("is employee detached: " + detached);
            if (detached) {
                System.out.println("merging employee");
                employee = entityManager.merge(employee);
            }
            //this will load/initialize employee entity
            employee.getName();
            employee.getDepartment();
            //this will load lazy phones field
            employee.getPhones().size();
            entityManager.detach(employee);
            //now employee is fully initialized
            System.out.println("employee initialized");
        }
        return employee;
    }


    private static Employee findEmployeeById(int id) {
        //also try with getReference
        //Employee employee = entityManager.getReference(Employee.class, id);
        Employee employee = entityManager.find(Employee.class, id);
        entityManager.detach(employee);
        return employee;
    }

    private static void persistEmployee() {
        Employee employee = new Employee("Joe", "IT",
                Phone.cell("111-111-1111"), Phone.work("222-222-2222"));
        entityManager.getTransaction().begin();
        entityManager.persist(employee);
        entityManager.getTransaction().commit();
        entityManager.detach(employee);
    }
}